//exceptions class


import 'package:strings_validator/validation/exceptions/exceptions.dart';
import 'package:strings_validator/validation/messages.dart';

class MainValidator {
  Map<String,String>? messages;
  Message erros=Message();

  MainValidator(Map? messages) {
    if (messages != null) {
      for (var param in messages.entries) {
        switch (param.key) {
          case 'required':
            erros.Empty = param.value;
            break;
          case 'min':
            erros.MinLength = param.value;
            break;
          case 'max':
            erros.MaxLength = param.value;
            break;
            case 'number':
            erros.Number = param.value;
            break;
            case 'alpha':
            erros.Alpha = param.value;
            break;
        }
      }
    }
  }
}

abstract class Validator extends MainValidator {
  Validator( messages) : super(messages);

  String? validate(String input);
}


class MaxLengthValidator extends Validator {
  int size;
  MaxLengthValidator(messages,{required this.size}):super(messages);

  @override
  String? validate(String input) {
    try{

      if (input.length>size) {
        throw LengthFormatException(erros.MaxLength);
      }
    } on BaseException catch (e) {
      return e.message;
    }
  }
}

class MinLengthValidator extends Validator {
  int size;
  MinLengthValidator(messages,{required this.size}):super(messages);

  @override
  String? validate(String input) {
    try{
      if (input.length<size) {
        throw  LengthFormatException(erros.MinLength);
      }
    } on BaseException catch (e) {
      return e.message;
    }
  }
}

class RequiredValidator extends Validator{
  RequiredValidator(messages) : super(messages);

  @override
  String? validate(String input) {
    try{
      if (input.isEmpty) {
        throw  EmptyException(erros.Empty);
      }
    }on BaseException catch (e) {
      return e.message;
    }
  }

}
class NumberValidator extends Validator{
  NumberValidator( messages) : super(messages);

  @override
  String? validate(String input) {
    try{
      if (!RegExp(r'^-?[0-9]+$').hasMatch(input)) {
        throw NumberException(erros.Number);
      }
    }on BaseException catch (e) {
      return e.message;
    }
  }

}
class AlphabetValidator extends Validator{
  AlphabetValidator(messages) : super(messages);

  @override
  String? validate(String input) {
    try{
      if (!RegExp(r'^[a-zA-Z_]*$').hasMatch(input)) {
        throw AlphabetException(erros.Alpha);
      }
    }on BaseException catch (e) {
      return e.message;
    }
  }
}

class EmailValidator extends Validator {
  EmailValidator( messages) : super(messages);

  @override
  String? validate(String input) {
    try {
      if (!RegExp(r"^[a-zA-Z0-9.a-zA-Z0-9.!#$%&'*+-/=?^_`{|}~]+@[a-zA-Z0-9]+\.[a-zA-Z]+").hasMatch(input)) {
        throw FormatException(erros.InvalidFormat);
      }
    } on BaseException catch (e) {
      return e.message;
    }
  }
}

class UpperCaseValidator extends Validator{
  UpperCaseValidator(messages) : super(messages);

  @override
  String? validate(String input) {
    if (!RegExp(r'^(?=.*?[A-Z])').hasMatch(input)) {
      throw UpperCasePasswordException(erros.ExistUpperCase);
    }
  }

}




class SpecialCharacterValidator extends Validator{
  SpecialCharacterValidator(messages) : super(messages);

  @override
  String? validate(String input) {

      if (!RegExp(r'^(?=.*?[!@#\$&*~])').hasMatch(input)) {
        throw SepcialCharacterPasswordException(erros.ExistSepecialChar);
    }
  }

}






//validation for string
//max:3
//min:2
//required
//number
//alpha
//email
//upperCase
//specialChar

extension StringValidation on String {
  String? validate(String param,{Map<String,String>?messages}) {
    String?errMessage;
    List<String>params=param.split('|').reversed.toList();
    for(var item in params){
      if(!item.contains(':')){
        switch(item){
          case 'required':
            errMessage= RequiredValidator(messages).validate(this)??errMessage;
            break;
          case 'number':
            errMessage=NumberValidator(messages).validate(this)??errMessage;
          break;
          case 'alpha':
            errMessage=AlphabetValidator(messages).validate(this)??errMessage;
          break;
          case 'email':
            errMessage=EmailValidator(messages).validate(this)??errMessage;
          break;
          case 'upperCase':
            errMessage=UpperCaseValidator(messages).validate(this)??errMessage;
          break;
          case 'specialChar':
            errMessage=SpecialCharacterValidator(messages).validate(this)??errMessage;
          break;
        }
      }else{
        List<String>items=item.split(':');
        switch(items[0]){
          case 'max':
            errMessage= MaxLengthValidator(messages,size:int.parse(items[1])).validate(this)??errMessage;
            break;
          case 'min':
            errMessage= MinLengthValidator(messages,size:int.parse(items[1])).validate(this)??errMessage;
            break;
        }
      }
    }
    return errMessage;
  }
}


//show the result of validation
// class ValidationResult{
//   List<String>errors;
//   String?lastError;
//   int errCount;
//
//   ValidationResult({List<String> ?errors, this.errCount=0}):errors=errors??[];
//
//   void evaluate(String? err){
//     if(err !=null){
//       lastError=err;
//       errors.add(err);
//       errCount++;
//     }
//   }
// }
//

